$DOORS::RADIANS_PER_DEGREE = 0.017444;
$DOORS::XY_UNIT_VECTORS = "0 0";
$DOORS::PLAYER_REACH = 4.5;
$DOORS::OPEN_INCREMENT = 0.02;
$DOORS::CLOSE_INCREMENT = 0.04;
$DOORS::OPEN_TIMER = 20;
$DOORS::CLOSE_TIMER = 10;
$DOORS::HOLD_TIMER = 5000;
$DOORS::MAX_ANGLE = 90;
$DOORS::MAX_SLIDE = 3.5;   //  ***new***
$DOORS::SLIDE_STEP = 0.05; //  ***new*** 
$DOORS::MODE_SWING = "swing"; //  ***new***
$DOORS::MODE_SLIDE = "slide"; //  ***new***

datablock AudioProfile(doorSwingOpenSndProfile)
{
   fileName = "~/data/sound/doors/doorswingopen.ogg";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(doorSwingClosedSndProfile)
{
   fileName = "~/data/sound/doors/doorswingclosed.ogg";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(doorSlamSndProfile)
{
   fileName = "~/data/sound/doors/doorslam.ogg";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(doorSlideSndProfile) // vvv ***new*** vvv
{
   fileName = "~/data/sound/doors/doorslide.ogg";
   description = AudioClose3d;
   preload = true;
}; // ^^^ ***new*** ^^^

datablock AudioProfile(doorThudSndProfile) // vvv ***new*** vvv
{
   fileName = "~/data/sound/doors/doorthud.ogg";
   description = AudioClose3d;
   preload = true;
}; // ^^^ ***new*** ^^^

datablock StaticShapeData(ADoor)
{
   category = "Doors";
   shapeFile = "~/data/shapes/doors/door.dts";
};

function ADoor::OnAdd(%theDatablock, %whichDoor)
{
  if(!%whichDoor.doorOpenFlag)
      %whichDoor.doorOpenFlag = false;
  if(!%whichDoor.rotationDirection)
      %whichDoor.rotationDirection = 1;
  if(!%whichDoor.maxOpenAngle)
      %whichDoor.maxOpenAngle = $DOORS::MAX_ANGLE; 
  if(!%whichDoor.maxSlideDistance)                      // vvv ***new*** vvv
  	%whichDoor.maxSlideDistance = $DOORS::MAX_SLIDE;
echo("a %whichDoor.mode:"@%whichDoor.mode);
  if(!%whichDoor.mode)                                  
  	%whichDoor.mode = $DOORS::MODE_SWING;            // ^^^ ***new*** ^^^
echo("b %whichDoor.mode:"@%whichDoor.mode);
  %whichDoor.currentRotation = 0;
  %whichDoor.originalRotation = 0;
  %whichDoor.partialTransform = "";
}


datablock StaticShapeData(BDoor)
{
   category = "Doors";
   shapeFile = "~/data/shapes/doors/door2.dts";
};

function BDoor::OnAdd(%theDatablock, %whichDoor)
{
	
  if(!%whichDoor.doorOpenFlag)
      %whichDoor.doorOpenFlag = false;
  if(!%whichDoor.rotationDirection)
      %whichDoor.rotationDirection = 1;
  if(!%whichDoor.maxOpenAngle)
      %whichDoor.maxOpenAngle = $DOORS::MAX_ANGLE; 
  if(!%whichDoor.maxSlideDistance)                      // vvv ***new*** vvv
  	%whichDoor.maxSlideDistance = $DOORS::MAX_SLIDE;
echo("a %whichDoor.mode:"@%whichDoor.mode);
  if(!%whichDoor.mode)                                  
  	%whichDoor.mode = $DOORS::MODE_SLIDE;            // ^^^ ***new*** ^^^
echo("b %whichDoor.mode:"@%whichDoor.mode);
  %whichDoor.currentRotation = 0;
  %whichDoor.originalRotation = 0;
  %whichDoor.partialTransform = "";
}






function serverCmdOperate(%client)
{
   %player = %client.player;
   %eye = %player.getEyeVector();
   %vec = vectorScale(%eye, $DOORS::PLAYER_REACH);
   %start = %player.getEyeTransform();
   %end = VectorAdd(%start,%vec);
   %found = ContainerRayCast (%start, %end, $TypeMasks::StaticObjectType, %player);
   if(%found)
      %found.getDataBlock().Operate(%found);
}

function ADoor::Operate(%theDatablock, %whichDoor)
{
  if (%whichDoor.doorOpenFlag == false)
  {
    switch$(%whichDoor.mode)                       // vvv ***new*** vvv
    {
      case $DOORS::MODE_SWING:
        %theDatablock.StartOpenSwing(%whichDoor);
      case $DOORS::MODE_SLIDE:
        %theDatablock.StartOpenSlide(%whichDoor);
      default:
        %theDatablock.StartOpenSwing(%whichDoor);
    }                                              // ^^^ ***new*** ^^^
  }
}

function ADoor::StartOpenSwing(%theDatablock, %whichDoor)
{
  %whichDoor.doorOpenFlag=true;
  %whichDoor.currentRotation = 0;
  %whichDoor.originalRotation = getword(%whichDoor.GetTransform(),6);
  %z_unit_vector = getword(%whichDoor.GetTransform(),5);
  if ( %z_unit_vector == 0 )
    %z_unit_vector = "1";
  %whichDoor.partialTransform = getwords(%whichDoor.GetTransform(),0,2) SPC $DOORS::XY_UNIT_VECTORS SPC %z_unit_vector;
  %whichDoor.openSndHandle = %whichDoor.playAudio(0,doorSwingOpenSndProfile);
  %theDatablock.IncrementSwing(%whichDoor);
}

function ADoor::StartOpenSlide(%theDatablock, %whichDoor)  // vvv ***new*** vvv
{
  %whichDoor.doorOpenFlag=true;
  %whichDoor.originalposition = %whichDoor.getposition();
  %transform = %whichDoor.getTransform();
  %rot = getWords(%transform, 3, 6);
  %matrix = VectorOrthoBasis(%rot);
  %whichDoor.openVector = getWords(%matrix, 6, 8);
  %whichDoor.closeVector = (-1*getword(%matrix,6)) SPC (-1*getword(%matrix,7)) SPC (-1*getword(%matrix,8));
  %whichDoor.openVector=vectorscale(%whichDoor.openVector, $DOORS::SLIDE_STEP);
  %whichDoor.closeVector=vectorscale(%whichDoor.closeVector, $DOORS::SLIDE_STEP);
  %whichDoor.openSndHandle = %whichDoor.playAudio(0,doorSlideSndProfile);
  %theDatablock.IncrementSlide(%whichDoor);
}    // ^^^ ***new*** ^^^

function ADoor::IncrementSwing(%theDatablock, %whichDoor)
{
  if ( %whichDoor.currentRotation < (%whichDoor.maxOpenAngle*$DOORS::RADIANS_PER_DEGREE))
  {
      %whichDoor.currentRotation  += ( $DOORS::OPEN_INCREMENT * %whichDoor.rotationDirection);
      %newrot = %whichDoor.originalRotation + %whichDoor.currentRotation;
      %whichDoor.settransform(%whichDoor.partialTransform SPC %newrot);
      %theDatablock.schedule($DOORS::OPEN_TIMER,"IncrementSwing", %whichDoor);
   }
  else
  {
    %whichDoor.stopAudio(%whichDoor.openSndHandle);
    %theDatablock.schedule($DOORS::HOLD_TIMER,"StartCloseSwing",%whichDoor);
  }
}


function ADoor::StartCloseSwing(%theDatablock, %whichDoor)
{
  %whichDoor.closeSndHandle = %whichDoor.playAudio(0,doorSwingClosedSndProfile);
  %theDatablock.schedule($DOORS::CLOSE_TIMER,"DecrementSwing", %whichDoor);
}

function ADoor::DecrementSwing(%theDatablock, %whichDoor)
{
  if ( %whichDoor.currentRotation > 0)
  {
    %whichDoor.currentRotation  -= ($DOORS::CLOSE_INCREMENT * %whichDoor.rotationDirection);
    %newrot = %whichDoor.originalRotation + %whichDoor.currentRotation;
    %whichDoor.settransform(%whichDoor.partialTransform SPC %newrot);
    %theDatablock.schedule($DOORS::CLOSE_TIMER,"DecrementSwing", %whichDoor);
  }
  else
  {
    %whichDoor.doorOpenFlag=false;
    %whichDoor.settransform(%whichDoor.partialTransform SPC %whichDoor.originalRotation);
    %whichDoor.stopAudio(%whichDoor.closeSndHandle);
    %whichDoor.playAudio(0,doorSlamSndProfile);
    %whichDoor.currentRotation = 0;
  }
}

//=====================================================



function BDoor::Operate(%theDatablock, %whichDoor)
{
  if (%whichDoor.doorOpenFlag == false)
  {
    switch$(%whichDoor.mode)                       // vvv ***new*** vvv
    {
      case $DOORS::MODE_SWING:
        %theDatablock.StartOpenSwing(%whichDoor);
      case $DOORS::MODE_SLIDE:
        %theDatablock.StartOpenSlide(%whichDoor);
      default:
        %theDatablock.StartOpenSwing(%whichDoor);
    }                                              // ^^^ ***new*** ^^^
  }
}



function BDoor::StartOpenSlide(%theDatablock, %whichDoor)  // vvv ***new*** vvv
{
  %whichDoor.doorOpenFlag=true;
  %whichDoor.originalposition = %whichDoor.getposition();
  %transform = %whichDoor.getTransform();
  %rot = getWords(%transform, 3, 6);
  %matrix = VectorOrthoBasis(%rot);
  %whichDoor.openVector = getWords(%matrix, 6, 8);
  %whichDoor.closeVector = (-1*getword(%matrix,6)) SPC (-1*getword(%matrix,7)) SPC (-1*getword(%matrix,8));
  %whichDoor.openVector=vectorscale(%whichDoor.openVector, $DOORS::SLIDE_STEP);
  %whichDoor.closeVector=vectorscale(%whichDoor.closeVector, $DOORS::SLIDE_STEP);
  %whichDoor.openSndHandle = %whichDoor.playAudio(0,doorSlideSndProfile);
  %theDatablock.IncrementSlide(%whichDoor);
}    // ^^^ ***new*** ^^^


function BDoor::IncrementSlide(%theDatablock, %whichDoor)  // vvv ***new*** vvv
{
  if (vectordist(%whichDoor.originalposition,%whichDoor.getposition()) < %whichDoor.maxSlideDistance)
  {
    %travel = vectoradd(%whichDoor.getposition(), %whichDoor.openVector);
    %whichDoor.settransform(%travel);
    %theDatablock.schedule($DOORS::OPEN_TIMER,"IncrementSlide", %whichDoor);
  }
  else
  {
    %whichDoor.stopAudio(%whichDoor.openSndHandle);
    %theDatablock.schedule($DOORS::HOLD_TIMER,"StartCloseSlide",%whichDoor);
  }
}    // ^^^ ***new*** ^^^


function BDoor::StartCloseSlide(%theDatablock, %whichDoor) // vvv ***new*** vvv
{
  %whichDoor.closeSndHandle = %whichDoor.playAudio(0,doorSlideSndProfile);
  %theDatablock.schedule($DOORS::CLOSE_TIMER,"DecrementSlide", %whichDoor);
}      // ^^^ ***new*** ^^^



function BDoor::DecrementSlide(%theDatablock, %whichDoor)  // vvv ***new*** vvv
{
  if (vectordist(%whichDoor.originalposition, %whichDoor.getposition()) > 0 )
  {
    %travel = vectoradd(%whichDoor.getposition(), %whichDoor.closeVector);
    %whichDoor.settransform(%travel);
    %theDatablock.schedule($DOORS::CLOSE_TIMER,"DecrementSlide", %whichDoor);
  }
  else
  {
    %whichDoor.doorOpenFlag=false;
    %whichDoor.settransform(%whichDoor.originalposition);
    %whichDoor.stopAudio(%whichDoor.closeSndHandle);
    %whichDoor.playAudio(0,doorThudSndProfile);
  }
}     